@@ -1,12 +1,14 @@
-use super::{ByteOrder, SigStr};
+use super::ByteOrder;
 use crate::rustbus_core;
 use rustbus_core::message_builder::{MarshalContext, UnmarshalContext};
 use rustbus_core::wire::marshal::traits::{Marshal, SignatureBuffer};
-use rustbus_core::wire::unixfd::UnixFd;
 use rustbus_core::wire::unmarshal::traits::Unmarshal;
 use rustbus_core::wire::validate_raw;
+use rustbus_core::wire::UnixFd;
 use std::sync::Arc;
 
+use arrayvec::ArrayVec;
+
 /// The body accepts everything that implements the Marshal trait (e.g. all basic types, strings, slices, Hashmaps,.....)
 /// And you can of course write an Marshal impl for your own datastrcutures
 #[derive(Debug)]
@@ -26,6 +28,107 @@ impl Default for MarshalledMessageBody {
     }
 }
 
+#[derive(Debug, PartialEq)]
+enum SigErr {
+    TooLong,
+    NonBaseDictKey,
+    ArraysTooNested,
+    StructsTooNested,
+    UnexpectedClosingParenthesis,
+    UnexpectedClosingBracket,
+    DictEntryNotInArray,
+    UnknownCharacter,
+    UnclosedStruct,
+    UnclosedDictEntry,
+    ArrayWithNoType,
+    TooManyTypesInDictEntry,
+}
+/*impl From<SigErr> for rustbus_core::signature::Error {
+    fn from(err: SigErr) -> Self {
+        use rustbus_core::signature::Error as RErr;
+        match err {
+            SigErr::TooLong => RErr::TooManyTypes,
+            SigErr::NonBaseDictKey => RErr::ShouldBeBaseType,
+            SigErr::ArraysTooNested | SigErr::StructsTooNested => RErr::NestingTooDeep,
+            SigErr::NonBaseDictKey
+
+
+        }
+    }
+}*/
+const MAX_NESTING_DEPTH: u8 = 32;
+
+fn validate_sig_str(sig: &str) -> Result<(), SigErr> {
+    if sig.len() > 255 {
+        return Err(SigErr::TooLong);
+    }
+    enum Nest {
+        Array,
+        DictEntry(u8), // u8 stores number of elements in dict
+        Struct(bool),  // bool stores if struct is non_empty
+    }
+    let mut stack = ArrayVec::<_, 64>::new();
+    let mut a_cnt = 0; //
+    let mut b_cnt = 0;
+    for c in sig.chars() {
+        match c {
+            'v' | 'a' | '{' | '(' if matches!(stack.last(), Some(&Nest::DictEntry(0))) => {
+                return Err(SigErr::NonBaseDictKey);
+            }
+            'a' if a_cnt >= MAX_NESTING_DEPTH => return Err(SigErr::ArraysTooNested),
+            'a' => {
+                stack.push(Nest::Array);
+                a_cnt += 1;
+                continue;
+            }
+            '(' if b_cnt >= MAX_NESTING_DEPTH => return Err(SigErr::StructsTooNested),
+            '(' => {
+                stack.push(Nest::Struct(false));
+                b_cnt += 1;
+                continue;
+            }
+            ')' if !matches!(stack.pop(), Some(Nest::Struct(true))) => {
+                return Err(SigErr::UnexpectedClosingParenthesis)
+            }
+            ')' => b_cnt -= 1,
+            '{' if !matches!(stack.last(), Some(&Nest::Array)) => {
+                return Err(SigErr::DictEntryNotInArray)
+            }
+            '{' => {
+                stack.push(Nest::DictEntry(0));
+                continue;
+            }
+            '}' if !matches!(stack.pop(), Some(Nest::DictEntry(2))) => {
+                return Err(SigErr::UnexpectedClosingBracket)
+            }
+            'v' | '}' | 'y' | 'b' | 'n' | 'q' | 'i' | 'u' | 'x' | 't' | 'd' | 's' | 'o' | 'g'
+            | 'h' => {}
+            _ => return Err(SigErr::UnknownCharacter),
+        }
+        while matches!(stack.last(), Some(&Nest::Array)) {
+            stack.pop();
+            a_cnt -= 1;
+        }
+        match stack.last_mut() {
+            Some(Nest::DictEntry(cnt)) if *cnt >= 2 => return Err(SigErr::TooManyTypesInDictEntry),
+            Some(Nest::DictEntry(cnt)) => *cnt += 1,
+            Some(Nest::Struct(non_empty)) => *non_empty = true,
+            _ => {}
+        }
+    }
+    if stack.is_empty() {
+        debug_assert_eq!(a_cnt, 0);
+        debug_assert_eq!(b_cnt, 0);
+        Ok(())
+    } else {
+        Err(match stack.last().unwrap() {
+            Nest::Struct(_) => SigErr::UnclosedStruct,
+            Nest::DictEntry(_) => SigErr::UnclosedDictEntry,
+            Nest::Array => SigErr::ArrayWithNoType,
+        })
+    }
+}
+
 impl MarshalledMessageBody {
     /// New messagebody with the default little endian byteorder
     #[inline]
@@ -97,19 +200,67 @@ impl MarshalledMessageBody {
     pub fn reserve(&mut self, additional: usize) {
         Arc::make_mut(&mut self.buf).reserve(additional)
     }
-    fn create_ctx(&mut self) -> MarshalContext {
-        MarshalContext {
-            buf: Arc::make_mut(&mut self.buf),
-            fds: &mut self.raw_fds,
-            byteorder: self.byteorder,
+    fn create_ctx2(&mut self) -> (MarshalContext, &mut SignatureBuffer) {
+        (
+            MarshalContext {
+                buf: Arc::make_mut(&mut self.buf),
+                fds: &mut self.raw_fds,
+                byteorder: self.byteorder,
+            },
+            &mut self.sig,
+        )
+    }
+    pub fn push_param_helper<F>(&mut self, push_fn: F) -> Result<(), rustbus_core::Error>
+    where
+        F: FnOnce(&mut MarshalContext, &mut SignatureBuffer) -> Result<(), rustbus_core::Error>,
+    {
+        let fd_pre_cnt = self.raw_fds.len();
+        let buf_pre_len = self.buf.len();
+        let sig_pre_len = self.sig.len();
+        let (mut ctx, sig) = self.create_ctx2();
+        match push_fn(&mut ctx, sig) {
+            Err(e) => {
+                self.raw_fds.truncate(fd_pre_cnt);
+                Arc::make_mut(&mut self.buf).truncate(buf_pre_len);
+                self.sig.truncate(sig_pre_len).unwrap();
+                Err(e)
+            }
+            Ok(()) => Ok(()),
+        }
+    }
+    fn push_param_core<P: Marshal>(
+        ctx: &mut MarshalContext,
+        sig: &mut SignatureBuffer,
+        p: P,
+    ) -> Result<(), rustbus_core::Error> {
+        p.marshal(ctx)?;
+        let pre_len = sig.len();
+        P::sig_str(sig);
+        if sig.len() > 255 {
+            let sig_err = rustbus_core::signature::Error::TooManyTypes;
+            let val_err = rustbus_core::ValError::InvalidSignature(sig_err);
+            Err(rustbus_core::Error::Validation(val_err))
+        } else if let Err(err) = validate_sig_str(&sig[pre_len..]) {
+            if !matches!(err, SigErr::ArraysTooNested | SigErr::StructsTooNested) {
+                panic!(
+                    "Invalid segment of signature added '{}'!: {:?}",
+                    &sig[pre_len..],
+                    err
+                );
+            }
+            let sig_err = rustbus_core::signature::Error::NestingTooDeep;
+            let val_err = rustbus_core::ValError::InvalidSignature(sig_err);
+            Err(rustbus_core::Error::Validation(val_err))
+        } else {
+            Ok(())
         }
     }
     /// Append something that is Marshal to the message body
     pub fn push_param<P: Marshal>(&mut self, p: P) -> Result<(), rustbus_core::Error> {
-        let mut ctx = self.create_ctx();
-        p.marshal(&mut ctx)?;
-        P::sig_str(&mut self.sig);
-        Ok(())
+        let push_fn = move |ctx: &mut MarshalContext, sig: &mut SignatureBuffer| {
+            Self::push_param_core(ctx, sig, p)
+        };
+        self.push_param_helper(push_fn)
     }
     /// Append two things that are Marshal to the message body
     pub fn push_param2<P1: Marshal, P2: Marshal>(
@@ -117,21 +268,11 @@ impl MarshalledMessageBody {
         p1: P1,
         p2: P2,
     ) -> Result<(), rustbus_core::Error> {
-        let mut ctx = self.create_ctx();
-        let pre_len = ctx.buf.len();
-        let pre_fds = ctx.fds.len();
-        let mut marshal = || {
-            p1.marshal(&mut ctx)?;
-            p2.marshal(&mut ctx)
+        let push_fn = move |ctx: &mut MarshalContext, sig: &mut SignatureBuffer| {
+            Self::push_param_core(ctx, sig, p1)?;
+            Self::push_param_core(ctx, sig, p2)
         };
-        if let Err(e) = (marshal)() {
-            ctx.buf.truncate(pre_len);
-            ctx.fds.truncate(pre_fds);
-            return Err(e);
-        }
-        P1::sig_str(&mut self.sig);
-        P2::sig_str(&mut self.sig);
-        Ok(())
+        self.push_param_helper(push_fn)
     }
 
     /// Append three things that are Marshal to the message body
@@ -141,23 +282,12 @@ impl MarshalledMessageBody {
         p2: P2,
         p3: P3,
     ) -> Result<(), rustbus_core::Error> {
-        let mut ctx = self.create_ctx();
-        let pre_len = ctx.buf.len();
-        let pre_fds = ctx.fds.len();
-        let mut marshal = || {
-            p1.marshal(&mut ctx)?;
-            p2.marshal(&mut ctx)?;
-            p3.marshal(&mut ctx)
+        let push_fn = move |ctx: &mut MarshalContext, sig: &mut SignatureBuffer| {
+            Self::push_param_core(ctx, sig, p1)?;
+            Self::push_param_core(ctx, sig, p2)?;
+            Self::push_param_core(ctx, sig, p3)
         };
-        if let Err(e) = (marshal)() {
-            ctx.buf.truncate(pre_len);
-            ctx.fds.truncate(pre_fds);
-            return Err(e);
-        }
-        P1::sig_str(&mut self.sig);
-        P2::sig_str(&mut self.sig);
-        P3::sig_str(&mut self.sig);
-        Ok(())
+        self.push_param_helper(push_fn)
     }
 
     /// Append four things that are Marshal to the message body
@@ -168,25 +298,13 @@ impl MarshalledMessageBody {
         p3: P3,
         p4: P4,
     ) -> Result<(), rustbus_core::Error> {
-        let mut ctx = self.create_ctx();
-        let pre_len = ctx.buf.len();
-        let pre_fds = ctx.fds.len();
-        let mut marshal = || {
-            p1.marshal(&mut ctx)?;
-            p2.marshal(&mut ctx)?;
-            p3.marshal(&mut ctx)?;
-            p4.marshal(&mut ctx)
+        let push_fn = move |ctx: &mut MarshalContext, sig: &mut SignatureBuffer| {
+            Self::push_param_core(ctx, sig, p1)?;
+            Self::push_param_core(ctx, sig, p2)?;
+            Self::push_param_core(ctx, sig, p3)?;
+            Self::push_param_core(ctx, sig, p4)
         };
-        if let Err(e) = (marshal)() {
-            ctx.buf.truncate(pre_len);
-            ctx.fds.truncate(pre_fds);
-            return Err(e);
-        }
-        P1::sig_str(&mut self.sig);
-        P2::sig_str(&mut self.sig);
-        P3::sig_str(&mut self.sig);
-        P4::sig_str(&mut self.sig);
-        Ok(())
+        self.push_param_helper(push_fn)
     }
 
     /// Append five things that are Marshal to the message body
@@ -198,41 +316,41 @@ impl MarshalledMessageBody {
         p4: P4,
         p5: P5,
     ) -> Result<(), rustbus_core::Error> {
-        let mut ctx = self.create_ctx();
-        let pre_len = ctx.buf.len();
-        let pre_fds = ctx.fds.len();
-        let mut marshal = || {
-            p1.marshal(&mut ctx)?;
-            p2.marshal(&mut ctx)?;
-            p3.marshal(&mut ctx)?;
-            p4.marshal(&mut ctx)?;
-            p5.marshal(&mut ctx)
+        let push_fn = move |ctx: &mut MarshalContext, sig: &mut SignatureBuffer| {
+            Self::push_param_core(ctx, sig, p1)?;
+            Self::push_param_core(ctx, sig, p2)?;
+            Self::push_param_core(ctx, sig, p3)?;
+            Self::push_param_core(ctx, sig, p4)?;
+            Self::push_param_core(ctx, sig, p5)
         };
-        if let Err(e) = (marshal)() {
-            ctx.buf.truncate(pre_len);
-            ctx.fds.truncate(pre_fds);
-            return Err(e);
-        }
-        P1::sig_str(&mut self.sig);
-        P2::sig_str(&mut self.sig);
-        P3::sig_str(&mut self.sig);
-        P4::sig_str(&mut self.sig);
-        P5::sig_str(&mut self.sig);
-        Ok(())
+        self.push_param_helper(push_fn)
     }
 
     /// Append any number of things that have the same type that is Marshal to the message body
     pub fn push_params<P: Marshal>(&mut self, params: &[P]) -> Result<(), rustbus_core::Error> {
-        for p in params {
-            self.push_param(p)?;
-        }
-        Ok(())
+        let push_fn = move |ctx: &mut MarshalContext, sig: &mut SignatureBuffer| {
+            for p in params {
+                Self::push_param_core(ctx, sig, p)?;
+            }
+            Ok(())
+        };
+        self.push_param_helper(push_fn)
     }
 
     /// Append something that is Marshal to the body but use a dbus Variant in the signature. This is necessary for some APIs
     pub fn push_variant<P: Marshal>(&mut self, p: P) -> Result<(), rustbus_core::Error> {
-        let mut ctx = self.create_ctx();
-        marshal_as_variant(p, &mut ctx)
+        let push_fn = move |ctx: &mut MarshalContext, sig: &mut SignatureBuffer| {
+            p.marshal_as_variant(ctx)?;
+            sig.push_static("v");
+            if sig.len() > 255 {
+                let sig_err = rustbus_core::signature::Error::TooManyTypes;
+                let val_err = rustbus_core::ValError::InvalidSignature(sig_err);
+                Err(rustbus_core::Error::Validation(val_err))
+            } else {
+                Ok(())
+            }
+        };
+        self.push_param_helper(push_fn)
     }
     /// Validate the all the marshalled elements of the body.
     pub fn validate(&self) -> Result<(), rustbus_core::wire::unmarshal::Error> {
@@ -254,7 +372,7 @@ impl MarshalledMessageBody {
     /// Create a parser to retrieve parameters from the body.
     #[inline]
     pub fn parser(&self) -> MessageBodyParser {
-        MessageBodyParser::new(&self)
+        MessageBodyParser::new(self)
     }
 }
 
@@ -428,20 +546,6 @@ impl<'ret, 'fds, 'body: 'ret + 'fds> MessageBodyParser<'body> {
     }
 }
 
-fn marshal_as_variant<T: Marshal>(
-    val: T,
-    ctx: &mut MarshalContext,
-) -> Result<(), rustbus_core::Error> {
-    let sig = T::signature();
-    let mut s_str = String::with_capacity(255);
-    sig.to_str(&mut s_str);
-    // SAFETY: assertion makes this safe
-    debug_assert!(SigStr::new(&s_str).is_ok());
-    let ss = unsafe { SigStr::new_no_val(&s_str) };
-    ss.marshal(ctx).unwrap();
-    val.marshal(ctx)
-}
-
 #[test]
 fn test_marshal_trait() {
     let mut body = MarshalledMessageBody::new();
@@ -610,3 +714,86 @@ fn test_marshal_trait() {
     assert_eq!(newmap2.get("a").unwrap().len(), 1);
     assert_eq!(*newmap2.get("a").unwrap().get("a").unwrap(), 4);
 }
+
+#[cfg(test)]
+mod tests {
+    use super::{validate_sig_str, SigErr};
+    use rustbus::params::validate_signature;
+
+    #[test]
+    fn test_validate_sig_str() {
+        for _ in 0..1000000 {
+            validate_sig_str("aaai").unwrap();
+            validate_sig_str("a{ii}").unwrap();
+            validate_sig_str("(ii)").unwrap();
+            validate_sig_str("(i)").unwrap();
+            validate_sig_str("a{i(i)}").unwrap();
+            /*assert_eq!(validate_sig_str("aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaai"), Err(SigErr::ArraysTooNested));
+            assert_eq!(validate_sig_str("(i(i(i(i(i(i(i(i(i(i(i(i(i(i(i(i(i(i(i(i(i(i(i(i(i(i(i(i(i(i(i(i(i)))))))))))))))))))))))))))))))))"), Err(SigErr::StructsTooNested));
+            assert_eq!(validate_sig_str(
+                "(((((((((((((((((((((((((((((((((y)))))))))))))))))))))))))))))))))"),
+                Err(SigErr::StructsTooNested)
+            );*/
+            assert_eq!(validate_sig_str("{ii}"), Err(SigErr::DictEntryNotInArray));
+            assert_eq!(
+                validate_sig_str("a{i(i})"),
+                Err(SigErr::UnexpectedClosingBracket)
+            );
+            assert_eq!(
+                validate_sig_str("()"),
+                Err(SigErr::UnexpectedClosingParenthesis)
+            );
+            assert_eq!(
+                validate_sig_str("a{}"),
+                Err(SigErr::UnexpectedClosingBracket)
+            );
+            assert_eq!(
+                validate_sig_str("a{iii}"),
+                Err(SigErr::TooManyTypesInDictEntry)
+            );
+            assert_eq!(validate_sig_str("((i)"), Err(SigErr::UnclosedStruct));
+            assert_eq!(
+                validate_sig_str("(i))"),
+                Err(SigErr::UnexpectedClosingParenthesis)
+            );
+            assert_eq!(validate_sig_str("a{{i}i}"), Err(SigErr::NonBaseDictKey));
+            assert_eq!(
+                validate_sig_str("a{ii}}"),
+                Err(SigErr::UnexpectedClosingBracket)
+            );
+            assert_eq!(validate_sig_str("!!!"), Err(SigErr::UnknownCharacter));
+            assert_eq!(
+                validate_sig_str(std::str::from_utf8(&[b'b'; 256]).unwrap()),
+                Err(SigErr::TooLong)
+            );
+            assert_eq!(validate_sig_str("(i)a"), Err(SigErr::ArrayWithNoType));
+        }
+    }
+    #[test]
+    fn test_rb_validate_signature() {
+        for _ in 0..1000000 {
+            validate_signature("aaai").unwrap();
+            validate_signature("a{ii}").unwrap();
+            validate_signature("(ii)").unwrap();
+            validate_signature("(i)").unwrap();
+            validate_signature("a{i(i)}").unwrap();
+            validate_signature("").unwrap();
+            /*validate_signature("aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaai").unwrap_err();
+            validate_signature("(i(i(i(i(i(i(i(i(i(i(i(i(i(i(i(i(i(i(i(i(i(i(i(i(i(i(i(i(i(i(i(i(i)))))))))))))))))))))))))))))))))").unwrap_err();
+            validate_signature("(((((((((((((((((((((((((((((((((y)))))))))))))))))))))))))))))))))").unwrap_err();
+            */
+            validate_signature("{ii}").unwrap_err();
+            validate_signature("a{i(i})").unwrap_err();
+            validate_signature("()").unwrap();
+            validate_signature("a{}").unwrap_err();
+            validate_signature("a{iii}").unwrap_err();
+            validate_signature("((i)").unwrap_err();
+            validate_signature("(i))").unwrap_err();
+            validate_signature("a{{i}i}").unwrap_err();
+            validate_signature("a{ii}}").unwrap_err();
+            validate_signature("!!!").unwrap_err();
+            validate_signature(std::str::from_utf8(&[b'b'; 256]).unwrap()).unwrap_err();
+            validate_signature("(i)a").unwrap_err();
+        }
+    }
+}
